@@ -96,78 +96,87 @@ AsComponentKind		as_component_kind_from_string (const gchar *kind_str);
 GType				as_component_get_type (void) G_GNUC_CONST;
 AsComponent*		as_component_new (void);
 AsComponent*		as_component_construct (GType object_type);
-gboolean			as_component_is_valid (AsComponent* cpt);
-gchar* 				as_component_to_string (AsComponent* cpt);
+gboolean			as_component_is_valid (AsComponent *cpt);
+gchar* 				as_component_to_string (AsComponent *cpt);
 
-AsComponentKind		as_component_get_kind (AsComponent* cpt);
-void				as_component_set_kind (AsComponent* cpt,
+AsComponentKind		as_component_get_kind (AsComponent *cpt);
+void				as_component_set_kind (AsComponent *cpt,
 										   AsComponentKind value);
 
-const gchar*		as_component_get_id (AsComponent* cpt);
-void				as_component_set_id (AsComponent* cpt,
+const gchar*		as_component_get_id (AsComponent *cpt);
+void				as_component_set_id (AsComponent *cpt,
 											const gchar* value);
 
-gchar**				as_component_get_pkgnames (AsComponent* cpt);
-void				as_component_set_pkgnames (AsComponent* cpt,
+const gchar*		as_component_get_origin (AsComponent *cpt);
+void				as_component_set_origin (AsComponent *cpt,
+											const gchar* origin);
+
+gchar**				as_component_get_pkgnames (AsComponent *cpt);
+void				as_component_set_pkgnames (AsComponent *cpt,
 												gchar** value);
 
-const gchar*		as_component_get_name (AsComponent* cpt);
-void				as_component_set_name (AsComponent* cpt,
+const gchar*		as_component_get_name (AsComponent *cpt);
+void				as_component_set_name (AsComponent *cpt,
 											const gchar* value);
-const gchar* 		as_component_get_name_original (AsComponent* cpt);
-void				as_component_set_name_original (AsComponent* cpt,
+const gchar* 		as_component_get_name_original (AsComponent *cpt);
+void				as_component_set_name_original (AsComponent *cpt,
 													const gchar* value);
 
-const gchar*		as_component_get_summary (AsComponent* cpt);
-void				as_component_set_summary (AsComponent* cpt,
+const gchar*		as_component_get_summary (AsComponent *cpt);
+void				as_component_set_summary (AsComponent *cpt,
 													const gchar* value);
 
-const gchar*		as_component_get_description (AsComponent* cpt);
-void				as_component_set_description (AsComponent* cpt,
+const gchar*		as_component_get_description (AsComponent *cpt);
+void				as_component_set_description (AsComponent *cpt,
 													const gchar* value);
 
-const gchar* 		as_component_get_project_license (AsComponent* cpt);
-void				as_component_set_project_license (AsComponent* cpt,
+const gchar* 		as_component_get_project_license (AsComponent *cpt);
+void				as_component_set_project_license (AsComponent *cpt,
 													const gchar* value);
 
-const gchar* 		as_component_get_project_group (AsComponent* cpt);
-void				as_component_set_project_group (AsComponent* cpt,
+const gchar* 		as_component_get_project_group (AsComponent *cpt);
+void				as_component_set_project_group (AsComponent *cpt,
 													const gchar* value);
 
-const gchar*		as_component_get_developer_name (AsComponent* cpt);
-void				as_component_set_developer_name (AsComponent* cpt,
+const gchar*		as_component_get_developer_name (AsComponent *cpt);
+void				as_component_set_developer_name (AsComponent *cpt,
 													const gchar* value);
 
-gchar**				as_component_get_compulsory_for_desktops (AsComponent* cpt);
-void				as_component_set_compulsory_for_desktops (AsComponent* cpt,
+gchar**				as_component_get_compulsory_for_desktops (AsComponent *cpt);
+void				as_component_set_compulsory_for_desktops (AsComponent *cpt,
 																gchar** value);
-gboolean			as_component_is_compulsory_for_desktop (AsComponent* cpt,
+gboolean			as_component_is_compulsory_for_desktop (AsComponent *cpt,
 																const gchar* desktop);
 
-gchar**				as_component_get_categories (AsComponent* cpt);
-void				as_component_set_categories (AsComponent* cpt,
+gchar**				as_component_get_categories (AsComponent *cpt);
+void				as_component_set_categories (AsComponent *cpt,
 												 gchar** value);
-void				as_component_set_categories_from_str (AsComponent* cpt,
+void				as_component_set_categories_from_str (AsComponent *cpt,
 												const gchar* categories_str);
 gboolean			as_component_has_category (AsComponent *cpt,
 												const gchar *category);
 
-GPtrArray*			as_component_get_screenshots (AsComponent* cpt);
-void				as_component_add_screenshot (AsComponent* cpt,
+GPtrArray*			as_component_get_screenshots (AsComponent *cpt);
+void				as_component_add_screenshot (AsComponent *cpt,
 												AsScreenshot* sshot);
 
-gchar**				as_component_get_keywords (AsComponent* cpt);
-void				as_component_set_keywords (AsComponent* cpt,
+gchar**				as_component_get_keywords (AsComponent *cpt);
+void				as_component_set_keywords (AsComponent *cpt,
 												gchar** value);
 
-const gchar*		as_component_get_icon (AsComponent* cpt);
-void				as_component_set_icon (AsComponent* cpt,
+const gchar*		as_component_get_icon (AsComponent *cpt);
+void				as_component_set_icon (AsComponent *cpt,
 											const gchar* value);
-const gchar*		as_component_get_icon_url (AsComponent* cpt);
-void				as_component_set_icon_url (AsComponent* cpt,
+const gchar*		as_component_get_icon_url_for_size (AsComponent *cpt,
+											int width,
+											int height);
+void				as_component_add_icon_url (AsComponent *cpt,
+											int width,
+											int height,
 											const gchar* value);
+GHashTable*			as_component_get_icon_urls (AsComponent *cpt);
 
-GPtrArray*			as_component_get_provided_items (AsComponent* cpt);
+GPtrArray*			as_component_get_provided_items (AsComponent *cpt);
 void				as_component_add_provided_item (AsComponent *cpt,
 										AsProvidesKind kind,
 										const gchar *value,
@@ -183,13 +192,13 @@ void				as_component_add_url (AsComponent *cpt,
 										AsUrlKind url_kind,
 										const gchar *url);
 
-GPtrArray*			as_component_get_releases (AsComponent* cpt);
-void				as_component_add_release (AsComponent* cpt,
+GPtrArray*			as_component_get_releases (AsComponent *cpt);
+void				as_component_add_release (AsComponent *cpt,
 												AsRelease* release);
 
 GPtrArray*			as_component_get_extends (AsComponent *cpt);
-void				as_component_add_extends (AsComponent* cpt,
-												const gchar* cpt_id);
+void				as_component_add_extends (AsComponent *cpt,
+												const gchar *cpt_id);
 
 void				as_component_add_language (AsComponent *cpt,
 												const gchar *locale,
@@ -200,6 +209,11 @@ GList*				as_component_get_languages (AsComponent *cpt);
 
 gchar*				as_component_to_xml (AsComponent *cpt);
 
+/* DEPRECATED */
+
+/* use G_GNUC_DEPRECATED on this later */
+const gchar*		as_component_get_icon_url (AsComponent *cpt);
+
 G_END_DECLS
 
 #endif /* __AS_COMPONENT_H */
